export interface CreateUserData {
  firstname: string;
  lastname: string;
  username: null | string;
  nickname: null | string;
  birthday: null | [number, number, number];
  email: null | string;
  role: 'student' | 'genericUser' | 'teacher' | 'schoolAdmin' | 'technicalAdmin';
}

export interface UserData extends CreateUserData {
  username: string;
}

export const randomInteger = (min: number, max: number): number => {
  expect(min).to.eq(Math.ceil(min));
  expect(max).to.eq(Math.ceil(max));
  expect(min).to.be.lessThan(max);
  return Math.round(Math.random() * (max - min)) + min;
};

export const randomChoice = <T>(options: T[]): T => options[randomInteger(0, options.length - 1)];

export const randomFirstName = (): string => randomChoice([
  'Emery',
  'Lilah',
  'Kyan',
  'Malaki',
  'Collin',
  'Elena',
  'Sanaa',
  'Clinton',
  'Rashad',
  'Justice',
  'Felipe',
  'Serena',
  'Walter',
  'Alissa',
  'Malik',
  'Madisyn',
  'Jocelynn',
  'Owen',
  'Marely',
  'Rylee',
]);

export const randomLastName = (): string => randomChoice([
  'Riley',
  'Schmidt',
  'Bradshaw',
  'Odom',
  'Clay',
  'Buchanan',
  'Riddle',
  'Tapia',
  'Le',
  'Saunders',
  'Mcbride',
  'Rhodes',
  'Colon',
  'Shannon',
  'Woodward',
  'Hooper',
  'Berger',
  'Terry',
  'Sullivan',
  'Conrad',
  'Small',
  'Marquez',
  'Castro',
  'Jackson',
  'Atkinson',
  'Rosales',
  'Webster',
  'Alvarado',
  'Galloway',
  'Mills',
  'Donovan',
  'Macias',
  'Nunez',
  'Cooper',
  'Gross',
  'Spence',
  'Kramer',
  'George',
  'Archer',
  'Frazier',
  'Vasquez',
  'Blankenship',
  'Gregory',
  'Whitney',
  'Barber',
  'Castaneda',
  'Perry',
  'Freeman',
  'Love',
  'Chandler',
  'Cooley',
  'Curry',
  'Holt',
  'Christensen',
  'Glenn',
  'Sutton',
  'Ramirez',
  'Stafford',
  'Holland',
  'White',
  'Potts',
  'Mendez',
  'Wilkinson',
  'Foley',
  'Escobar',
  'Johnston',
  'Black',
  'Hatfield',
  'Avila',
  'Santana',
  'Crane',
  'Maxwell',
  'Kerr',
  'Shepherd',
  'Martinez',
  'Molina',
  'Gamble',
  'Ryan',
  'Sawyer',
  'Mcconnell',
  'Clements',
  'Diaz',
  'Bonilla',
  'Hanna',
  'Levine',
  'Terrell',
  'Sexton',
  'Gill',
  'Duran',
  'Wu',
  'Stout',
  'Graham',
  'Newman',
  'Aguirre',
  'Zuniga',
  'Hancock',
  'Hines',
  'Jennings',
  'Reyes',
  'Rasmussen',
]);

export const randomString = (length: number): string => {
  const alphabet = 'abcdefghijklmnopqrstuvwxyz';
  let result = '';
  for (let i = 0; i < length; i++) {
    let char = alphabet.charAt(Math.floor(Math.random() * alphabet.length));
    if (Math.random() < 0.5) {
      char = char.toUpperCase();
    }
    result += char;
  }
  return result;
};

export const randomUserData = (options?: {nickname: boolean}): UserData => {
  // TODO: Would be good to fuzz these values
  const firstname = randomFirstName();
  const lastname = randomLastName();
  const nickname = options?.nickname ? randomString(10) : null;
  const username = randomString(15);
  return {
    firstname,
    lastname,
    username,
    nickname,
    birthday: [
      randomInteger(1960, 2000),
      randomInteger(1, 12),
      randomInteger(1, 27),
    ],
    email: null,
    // This doesn't work, currently, as the mail address is sent as
    //  `primaryMail` to the server, which seems to be wrong.
    // email: `${username}@example.com`,
    role: 'student',
  };
};
